import jp.jig.jiglet.*;

public class SimpleGame extends Jiglet {
	static final int FIELD_W = 8;
	static final int FIELD_H = 13;
	static final int BLOCK_SIZE = 15;
	
	static final int STATE_NORMAL = 0;
	static final int STATE_DROP = 10;
	static final int STATE_DELETE = 20;
	
	static final int DROP_FRAME = 3;
	static final int DELETE_FRAME = 20;
	
	static int[][] blockColor;
	static int[][] blockFlag;
	static int maxColor;
	
	static int dropColor1;
	static int dropColor2;
	static int dropX;
	static int dropY;
	static int dropVec;
	
	static int lastX;
	static int lastY;
	static int lastVec;
	
	static int level;
	static int score;
	static int delBlocks;
	static int chain;
	static int maxChain;
	
	static int highScore;
	
	static final int[] DX = new int[] {1, 0, -1, 0};
	static final int[] DY = new int[] {0, 1, 0, -1};
	
	static final int FIELD_DX = 20;
	static final int FIELD_DY = 20;
	static final int FIELD_DW = BLOCK_SIZE * FIELD_W;
	static final int FIELD_DH = BLOCK_SIZE * (FIELD_H - 2);
	static final int STATUS_DX = FIELD_DX + FIELD_DW + BLOCK_SIZE;
	
	public void main() {
		if (blockColor == null) {
			initialize();
			createBlock();
			
			byte[] dat = readSP(0, 4);
			highScore = (dat[0] & 0xFF) << 24;
			highScore |= (dat[1] & 0xFF) << 16;
			highScore |= (dat[2] & 0xFF) << 8;
			highScore |= (dat[3] & 0xFF) << 0;
		}
		setTitle("SimpleGame");
		clearKey();
		
		long loopTime = currentTime();
		long dropTime = currentTime();
		setSoftLabel("I");
		for (;;) {
			repaint();
			int def = 40 - (int)(currentTime() - loopTime);
			sleep(def);
			loopTime = currentTime();
			
			int key = getKey();
			
			if (key == KEY_IAPP || key == KEY_SOFT) {
				terminate();
			}
			
			if (key == KEY_SELECT) {
				rotateBlock(-1);
			}
			if (key == KEY_UP) {
				rotateBlock(1);
			}
			
			if (key == KEY_LEFT) {
				moveBlock(-1, 0);
			}
			if (key == KEY_RIGHT) {
				moveBlock(1, 0);
			}
			int dt = 1000 - level * 20;
			if (dt < 100) {
				dt = 100;
			}
			if (isKeyPressed(KEY_DOWN) || (currentTime() - dropTime) >= dt) {
				if (!moveBlock(0, 1)) {
					//n
					blockColor[dropX][dropY] = dropColor1;
					blockColor[dropX + DX[dropVec]][dropY + DY[dropVec]] = dropColor2;
					dropColor1 = 0;
					dropColor2 = 0;
					//Ƃ
					for (int ch = 1;; ch++) {
						dropField();
						lastX = -1;
						repaint();
						sleep(200);
						if (!deleteField(ch)) {
							break;
						}
						lastX = -1;
						repaint();
						sleep(200);
					}
					if (chain > maxChain) {
						maxChain = chain;
					}
					
					//Q[I[o[`FbN
					boolean gameOver = false;
					for (int x = 0; x < FIELD_W; x++) {
						if (blockColor[x][1] > 0) {
							gameOver = true;
						}
					}
					if (gameOver) {
						dropColor1 = -2;
						lastX = -1;
						repaint();
						clearKey();
						
						if (score > highScore) {
							highScore = score;
							byte[] dat = new byte[] {
								(byte)(highScore >> 24),
								(byte)(highScore >> 16),
								(byte)(highScore >> 8),
								(byte)(highScore),
							};
							writeSP(0, dat);
						}
						
						while (getKey() < 0) {
							sleep(100);
						}
						clearKey();
						initialize();
						createBlock();
					} else {
						createBlock();
						sleep(100);
					}
					clearKey();
				}
				dropTime = currentTime();
			}
		}
	}
	
	void createBlock() {
		dropColor1 = random(maxColor) + 1;
		dropColor2 = random(maxColor) + 1;
		dropX = FIELD_W / 2 - 1;
		dropY = 1;
		dropVec = 0;
		
		lastX = -1;
		lastY = -1;
		lastVec = -1;
	}
	
//----vZ----------------------------------------------------------
	//ubNړ
	boolean moveBlock(int dx, int dy) {
		int x1 = dropX + dx;
		int y1 = dropY + dy;
		int x2 = dropX + dx + DX[dropVec];
		int y2 = dropY + dy + DY[dropVec];
		
		if (x1 < 0 || x1 >= FIELD_W || x2 < 0 || x2 >= FIELD_W || y1 < 0 || y2 < 0 || y1 >= FIELD_H || y2 >= FIELD_H) {
			return false;
		}
		if (blockColor[x1][y1] > 0 || blockColor[x2][y2] > 0) {
			return false;
		}
		dropX += dx;
		dropY += dy;
		return true;
	}
	void rotateBlock(int d) {
		int def = (dropVec + d + 4) % 4;;
		int x2 = dropX + DX[def];
		int y2 = dropY + DY[def];
		
		if (x2 < 0 || x2 >= FIELD_W || y2 < 0 || y2 >= FIELD_H) {
			return;
		}
		if (blockColor[x2][y2] > 0) {
			return;
		}
		dropVec = def;
	}
	
	//ubN
	void dropField() {
		for (int x = 0; x < FIELD_W; x++) {
			int def = 0;
			for (int y = FIELD_H - 1; y > 0; y--) {
				if (blockColor[x][y] == 0) {
					def++;
				} else if (def > 0) {
					blockColor[x][y + def] = blockColor[x][y];
					blockColor[x][y] = 0;
				}
			}
		}
	}
	boolean deleteField(int ch) {
		for (int x = 0; x < FIELD_W; x++) {
			for (int y = 0; y < FIELD_H; y++) {
				blockFlag[x][y] = 0;
			}
		}
		for (int x = 0; x < FIELD_W; x++) {
			for (int y = 0; y < FIELD_H; y++) {
				if (blockColor[x][y] > 0) {
					if (x > 0 && x < FIELD_W - 1) {
						if (blockColor[x][y] == blockColor[x - 1][y] && blockColor[x][y] == blockColor[x + 1][y]) {
							blockFlag[x][y] = 1;
							blockFlag[x - 1][y] = 1;
							blockFlag[x + 1][y] = 1;
						}
					}
					if (y > 0 && y < FIELD_H - 1) {
						if (blockColor[x][y] == blockColor[x][y - 1] && blockColor[x][y] == blockColor[x][y + 1]) {
							blockFlag[x][y] = 1;
							blockFlag[x][y - 1] = 1;
							blockFlag[x][y + 1] = 1;
						}
					}
				}
			}
		}
		int num = 0;
		for (int x = 0; x < FIELD_W; x++) {
			for (int y = 0; y < FIELD_H; y++) {
				if (blockFlag[x][y] > 0) {
					blockColor[x][y] = 0;
					num++;
				}
			}
		}
		if (num > 0) {
			chain = ch;
		}
		addScore(num);
		return num > 0;
	}
	void addScore(int n) {
		score += n * level * chain * chain;
		delBlocks += n;
		level = (delBlocks / 10) + 1;
	}
	
	
	//
	void initialize() {
		blockColor = new int[FIELD_W][FIELD_H];
		blockFlag = new int[FIELD_W][FIELD_H];
		maxColor = 4;
		
		level = 1;
		score = 0;
		delBlocks = 0;
		chain = 0;
		maxChain = 0;
		
		lastX = -1;
	}
	
//----`----------------------------------------------------------
	public void paint(boolean allDrawFlag) {
		setFont(0);
		if (blockColor != null) {
			if (dropColor1 <= 0 || allDrawFlag || lastX < 0) {
				setColor(0, 0, 0);
				fillRect(0, 0, DW, DH);
				
				for (int x = 0; x < FIELD_W; x++) {
					for (int y = 1; y < FIELD_H; y++) {
						if (blockColor[x][y] > 0) {
							int bx = FIELD_DX + BLOCK_SIZE * x;
							int by = FIELD_DY + BLOCK_SIZE * y;
							drawBlock(blockColor[x][y], bx, by);
						}
					}
				}
				if (dropColor1 > 0) {
					int bx = FIELD_DX + BLOCK_SIZE * dropX;
					int by = FIELD_DY + BLOCK_SIZE * dropY;
					drawBlock(dropColor1, bx, by);
					bx = FIELD_DX + BLOCK_SIZE * (dropX + DX[dropVec]);
					by = FIELD_DY + BLOCK_SIZE * (dropY + DY[dropVec]);
					drawBlock(dropColor2, bx, by);
				}
				
				if (dropColor1 == -2) {		//gameover
					int gx = FIELD_DX + (FIELD_DW - 70) / 2;
					int gy = FIELD_DY + FIELD_DH / 2 - 15;
					setColor(0, 0, 0);
					fillRect(gx, gy, 61, 30);
					setColor(255, 255, 255);
					drawString("Game Over", gx + 8, gy + 13);
					if (score > highScore) {
						drawString("HighScore!", gx + 5, gy + 28);
					}
				}
				
				setColor(255, 255, 255);
				int dy = FIELD_DY + 50;
				drawString("LEVEL:" + level, STATUS_DX, dy);
				dy += 20;
				
				drawString("SCORE:" + score, STATUS_DX, dy);
				dy += 20;
				
				drawString("BLOCKS:" + delBlocks, STATUS_DX, dy);
				dy += 20;
				
				drawString("MAXCHAIN:" + maxChain, STATUS_DX, dy);
				dy += 20;
				
				drawString("HIGH:" + highScore, STATUS_DX, dy);
				dy += 20;
				
				if (chain > 1) {
					drawString("" + chain + "CHAIN", STATUS_DX, dy);
				}
				lastX = dropX;
				lastY = dropY;
				lastVec = dropVec;
			} else {
				if (lastX >= 0) {
					int bx = FIELD_DX + BLOCK_SIZE * lastX;
					int by = FIELD_DY + BLOCK_SIZE * lastY;
					setColor(getColor(0, 0, 0));
					fillRect(bx, by, BLOCK_SIZE, BLOCK_SIZE);
					bx = FIELD_DX + BLOCK_SIZE * (lastX + DX[lastVec]);
					by = FIELD_DY + BLOCK_SIZE * (lastY + DY[lastVec]);
					fillRect(bx, by, BLOCK_SIZE, BLOCK_SIZE);
				}
				
				int bx = FIELD_DX + BLOCK_SIZE * dropX;
				int by = FIELD_DY + BLOCK_SIZE * dropY;
				drawBlock(dropColor1, bx, by);
				bx = FIELD_DX + BLOCK_SIZE * (dropX + DX[dropVec]);
				by = FIELD_DY + BLOCK_SIZE * (dropY + DY[dropVec]);
				drawBlock(dropColor2, bx, by);
				
				lastX = dropX;
				lastY = dropY;
				lastVec = dropVec;
			}
			setColor(255, 255, 255);
			drawRect(FIELD_DX - 1, FIELD_DY - 1 + BLOCK_SIZE * 2, FIELD_DW + 1, FIELD_DH + 1);
		}
	}
	
	static int[] colorList = new int[] {
		getColor(0, 0, 0),			getColor(0, 0, 0),				getColor(0, 0, 0),
		getColor(0, 0, 255),		getColor(150, 150, 255),		getColor(0, 0, 200),
		getColor(0, 255, 0),		getColor(150, 255, 150),		getColor(0, 200, 0),
		getColor(255, 0, 0),		getColor(255, 255, 150),		getColor(200, 0, 0),
		getColor(0, 255, 255),		getColor(150, 255, 255),		getColor(0, 200, 200),
		getColor(255, 0, 255),		getColor(255, 150, 255),		getColor(200, 0, 200),
		getColor(255, 255, 0),		getColor(255, 255, 150),		getColor(200, 0, 0),
		getColor(255, 255, 255),	getColor(255, 255, 255),		getColor(200, 200, 200),
	};
	
	static void drawBlock(int col, int x, int y) {
		setColor(colorList[col * 3]);
		fillRect(x, y, BLOCK_SIZE, BLOCK_SIZE);
		
		setColor(colorList[col * 3 + 1]);
		drawLine(x, y, x + BLOCK_SIZE  - 1, y);
		drawLine(x, y, x, y + BLOCK_SIZE - 1);
		
		setColor(colorList[col * 3 + 2]);
		drawLine(x, y + BLOCK_SIZE - 1, x + BLOCK_SIZE - 1, y + BLOCK_SIZE - 1);
		drawLine(x + BLOCK_SIZE - 1, y, x + BLOCK_SIZE - 1, y + BLOCK_SIZE - 1);
	}
}
